@@ -5,9 +5,9 @@ use warnings;
 
 require 5.006;
 
-use IO::Compress::Zip 2.057 qw(:all);
-use IO::Compress::Base::Common  2.057 ();
-use IO::Compress::Adapter::Deflate 2.057 ;
+use IO::Compress::Zip 2.059 qw(:all);
+use IO::Compress::Base::Common  2.059 ();
+use IO::Compress::Adapter::Deflate 2.059 ;
 
 use Fcntl;
 use File::Spec;
@@ -18,7 +18,7 @@ require Exporter ;
 our ($VERSION, @ISA, @EXPORT_OK, %EXPORT_TAGS, $SimpleZipError);
 
 $SimpleZipError= '';
-$VERSION = "0.005";
+$VERSION = "0.006";
 
 @ISA = qw(Exporter);
 @EXPORT_OK = qw( $SimpleZipError ) ;
@@ -40,6 +40,9 @@ our %PARAMS = (
         'textflag'      => [IO::Compress::Base::Common::Parse_boolean,   0],
         'storelinks'    => [IO::Compress::Base::Common::Parse_boolean,   0],
         #'storedirs'    => [IO::Compress::Base::Common::Parse_boolean,   0],
+        'encode'       => [IO::Compress::Base::Common::Parse_any,        undef],
+        #'extrafieldlocal'  => [IO::Compress::Base::Common::Parse_any,    undef],
+        #'extrafieldcentral'=> [IO::Compress::Base::Common::Parse_any,    undef],
         
 
         # Zlib
@@ -57,6 +60,7 @@ our %PARAMS = (
         'verbosity'     => [IO::Compress::Base::Common::Parse_boolean,   0],
     );
 
+
 sub _ckParams
 {
     my $got = shift || IO::Compress::Base::Parameters::new();
@@ -64,7 +68,7 @@ sub _ckParams
        
     $got->parse(\%PARAMS, @_) 
         or _myDie("Parameter Error: " . $got->getError())  ;
-
+    
     if ($top)
     {
         for my $opt ( qw(name comment) )
@@ -85,6 +89,13 @@ sub _ckParams
                 if $got->parsed($opt);
         }
     }
+    
+    my $e = $got->getValue("encode");
+    if (defined $e)
+    {
+        _myDie("Encode::find_encoding not found") if ! defined &Encode::find_encoding;
+        _myDie("Unknown Encoding '$e'")           if ! defined Encode::find_encoding($e) ;
+    }
 
     return $got;
 }
@@ -194,8 +205,16 @@ sub close
     
     if ($self->{FilesWritten})
     {
-        defined $self->{Zip} && $self->{Zip}->flush() && $self->{Zip}->close()        
-            or return 0 ;
+         if(defined $self->{Zip})
+         {       
+             if ($self->{Raw})   
+             {
+                bless $self->{Zip}, "IO::Compress::Zip";                
+             }
+             
+            $self->{Zip}->flush() || return 0;
+            $self->{Zip}->close() || return 0;
+         }
     }
       
     1;
@@ -206,12 +225,6 @@ sub _newStream
     my $self = shift;
     my $filename = shift ;
     my $options =  shift;
-    my %user_options =  @_ ;
-    
-    while( my ($name, $value) =  each %user_options)
-    {
-        $options->setValue(lc $name, $value);
-    }
 
     if (defined $filename)
     {
@@ -225,6 +238,7 @@ sub _newStream
     # Archive::Zip::SimpleZip handles canonical    
     $options->setValue(canonicalname => 0);
 
+    
     if (! defined $self->{Zip}) {
         $self->{Zip} = IO::Compress::Base::Common::createSelfTiedObject('IO::Compress::Zip', \$SimpleZipError);    
         $self->{Zip}->_create($options, $self->{FH})        
@@ -234,7 +248,7 @@ sub _newStream
         $self->{Zip}->_newStream($options)
             or die "$SimpleZipError";
     }
-
+    
     ++ $self->{FilesWritten} ;
     
     return 1;
@@ -271,10 +285,10 @@ sub _stdPreq
     return 0 
         if $self->{Error} ; 
             
-    return $self->_setError(0, "zip file closed") 
+    return $self->_setError(0, "Zip file closed") 
         if ! $self->{Open} ;
             
-    return $self->_setError(0, "raw mode enabled") 
+    return $self->_setError(0, "openMember filehandle already open") 
         if  $self->{Raw};
            
      return 1;    
@@ -296,6 +310,9 @@ sub add
     my $options =  $self->{Opts}->clone();
         
     my $got = _ckParams($options, 0, @_);
+    
+    # Force Encode off .
+    $got->setValue('encode', undef);
 
     my $isLink = $got->getValue('storelinks') && -l $filename ;
     
@@ -326,6 +343,7 @@ sub add
         binmode $fh;
 
         my $data; 
+        my $last ;
         while ($fh->read($data, 1024 * 16))
         {
             $self->{Zip}->write($data);
@@ -347,7 +365,6 @@ sub addString
 
     $self->_stdPreq or return 0 ;
 
-
     my $options =  $self->{Opts}->clone();
         
     my $got = _ckParams($options, 0, @_);
@@ -355,13 +372,58 @@ sub addString
     _myDie("Missing 'Name' parameter in addString")
         if ! $got->parsed("name");
 
-
     $self->_newStream(undef, $got);
     $self->{Zip}->write($string);    
     
     return 1;            
 }
 
+sub openMember
+{
+    my $self    = shift;     
+
+    $self->_stdPreq or return undef ;
+
+    my $options =  $self->{Opts}->clone();
+        
+    my $got = _ckParams($options, 0, @_);
+    
+    _myDie("Missing 'Name' parameter in openMember")
+        if ! $got->parsed("name");
+
+    $self->_newStream(undef, $got);
+
+    $self->{Raw} = 1;
+    *{ $self->{Zip} }->{SZ} = \$self->{Raw} ;
+    
+    return bless $self->{Zip}, "Archive::Zip::SimpleZip::Handle" ;          
+}
+
+{
+    package Archive::Zip::SimpleZip::Handle ;
+    
+    our (@ISA);   
+    @ISA = qw(IO::Compress::Zip);
+           
+    sub DESTROY
+    {
+        my $self = shift ;  
+        $self->close();       
+        undef $self;
+    } 
+    
+    sub close
+    {
+        my $self = shift ;
+        ${ *$self->{SZ} } = 0;
+        *$self->{SZ} = undef ;
+        bless $self, "IO::Compress::Zip" ;
+        1;
+    }
+
+
+    *CLOSE    = \&close;
+}
 
 1;
 
@@ -381,21 +443,34 @@ Archive::Zip::SimpleZip - Create Zip Archives
     $z->add("/some/file1.txt");
     $z->addString("some text", Name => "myfile");
 
+    $fh = $z->openMember(Name => "mydata1.txt");
+    print $fh "some data" ;
+    close $fh;
+
+    $fh = $z->openMember(Name => "mydata2.txt");
+    $fh->print("some data") ;
+    $fh->close() ;
+
     $z->close();
 
 =head1 DESCRIPTION
 
-Archive::Zip::SimpleZip is a module that allows the creation of Zip archives. 
-It doesn't allow modification of existing zip archives - it just writes zip archives from scratch.
+Archive::Zip::SimpleZip is a module that allows the creation of Zip
+archives. 
 
-There are a few methods available in Archive::Zip::SimpleZip, and quite a few options, 
-but for the most part all you need to know is how to create a Zip object 
-and how to add a file to the zip archive. 
-Below is a typical example of how it is used
+The module allows Zip archives to be written to a named file, a filehandle
+or stored in-memory.
+
+There are a small number methods available in Archive::Zip::SimpleZip, and
+quite a few options, but for the most part all you need to know is how to
+create a Zip archive and how to add a file to it. 
+
+Below is an example of how this module is used to add the two files
+"file1.txt" and "file2.txt" to the zip file called "my1.zip".
 
     use Archive::Zip::SimpleZip qw($SimpleZipError) ;
 
-    my $z = new Archive::Zip::SimpleZip "my.zip"
+    my $z = new Archive::Zip::SimpleZip "my1.zip"
         or die "Cannot create zip file: $SimpleZipError\n" ;
 
     $z->add("/some/file1.txt");
@@ -403,6 +478,38 @@ Below is a typical example of how it is used
 
     $z->close();
 
+The data written to a zip archive doesn’t need to come from the filesystem.
+You can also write string data directly to the zip archive using the
+C<addString> method, like this
+
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $z = new Archive::Zip::SimpleZip "my2.zip"
+        or die "Cannot create zip file: $SimpleZipError\n" ;
+
+    $z->addString($myData, Name => "file2.txt");
+
+    $z->close();
+
+Alternatively you can use the C<openMember> option to get a filehandle that
+allows you to write directly to the zip archive member using standard Perl
+file output functions, like C<print>. 
+
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $z = new Archive::Zip::SimpleZip "my3.zip"
+        or die "Cannot create zip file: $SimpleZipError\n" ;
+
+    my $fh = $z->openMember(Name => "file3.txt");
+
+    $fh->print("some data");
+
+    print $fh "more data" ;
+
+    $fh->close() ; 
+    # can also use close $fh;
+
+    $z->close();
 
 =head2 Constructor
 
@@ -410,31 +517,37 @@ Below is a typical example of how it is used
      $z = new Archive::Zip::SimpleZip \$buffer [, OPTIONS] ;
      $z = new Archive::Zip::SimpleZip $filehandle [, OPTIONS] ;
 
-The constructor takes one mandatory parameter along with zero or more optional patameters.
+The constructor takes one mandatory parameter along with zero or more
+optional parameters.
 
-The mandatory parameter controls where the zip archive is written. 
-This can be any of the the following
+The mandatory parameter controls where the zip archive is written.  This
+can be any of the following
 
 =over 5
 
-=item * A File
+=item * Output to a File
 
-When SimpleZip is passed a string, it will write the zip archive to the filename stored in the string.
+When SimpleZip is passed a string, it will write the zip archive to the
+filename stored in the string.
 
-=item * A String
+=item * Output to a String
 
-When SimpleZip is passed a string reference, it will write the zip archive to the string.
+When SimpleZip is passed a string reference, like C<\$buffer>, it will
+write the zip archive into that string.
 
-=item * A Filehandle
+=item * Output to a Filehandle
 
-When SimpleZip is passed a filehandle, it will write the zip archive to that filehandle. 
+When SimpleZip is passed a filehandle, it will write the zip archive to
+that filehandle. 
 
-Use the string '-' to write the zip archive to standard output (Note - this will also enable the C<Stream> option). 
+Use the string '-' to write the zip archive to standard output (Note - this
+will also enable the C<Stream> option). 
 
 
 =back
 
-See L</Options> for a list of the optional parameters that can be specified when calling the constructor.
+See L</Options> for a list of the optional parameters that can be specified
+when calling the constructor.
 
 =head2 Methods
 
@@ -442,16 +555,16 @@ See L</Options> for a list of the optional parameters that can be specified when
 
 =item $z->add($filename [, OPTIONS])
 
-The C<add> method writes the contents of the filename stored in C<$filename> to the zip archive.
+The C<add> method writes the contents of the filename stored in
+C<$filename> to the zip archive.
  
-
-Currenly the module supports these file types.
+The following file types are supported.
 
 =over 5
 
 =item * Standard files
 
-The contents of the the file are written to the zip archive. 
+The contents of the file is written to the zip archive. 
 
 =item * Directories
 
@@ -459,42 +572,77 @@ The directory name is stored in the zip archive.
 
 =item * Symbolic Links
 
-By default this module will store the contents of the file that the symbolic link refers to.
-It is possible though to store the symbolic link itself by setting the C<StoreLink> option to 1.
+By default this module will store the contents of the file that the
+symbolic link refers to.  To store the symbolic link itself set the
+C<StoreLink> option to 1.
 
 
 =back
 
-By default the name of the entry created in the zip archive will be based on 
-the value of the $filename parameter. 
-See L</File Naming Options> for more details.  
+By default the name of the member created in the zip archive will be
+derived from the value of the C<$filename> parameter.  See L</File Naming
+Options> for more details.  
 
 See L</Options> for a full list of the options available for this method.
 
-Returns 1 if the file was added, or 0. Check the $SimpleZipError for a message.
+Returns 1 if the file was added, or 0. Check the $SimpleZipError for a
+message.
 
-=item $z->addString($string, Name => "whatever", [, OPTIONS]) 
+=item $z->addString($string, Name => "whatever" [, OPTIONS]) 
 
-The addString method writes <$string> to the zip archive. The Name option must be supplied.
+The C<addString> method writes <$string> to the zip archive. The C<Name>
+option I<must> be supplied.
 
 See L</Options> for the options available for this method.
 
-Returns 1 if the file was added, or 0. Check the $SimpleZipError for a message.
+Returns 1 if the file was added, or 0. Check the $SimpleZipError for a
+message.
+
+=item my $fh = $z->openMember(Name => "abc" [, OPTIONS]);
+
+This option returns a filehandle (C<$fh>)that allows you to write directly
+to a zip member. The C<Name> option I<must> be supplied.  See L</Options>
+for the options available for this method.
+
+The filehandle returned works just like a standard Perl filehandle, so all
+the standard file output operators, like C<print>, are available to write
+to the zip archive.
+
+When you have finished writing data to the member, close the filehandle by
+letting it go out of scope or explicitly  using either
+
+    $fh->close()
+    close $fh;
+
+In addition, if the enclosing zip object is closed whilst a filehandle is
+still open for a zip member, it will be closed automatically.
+
+Note that while a zip member has been opened with C<openMember>, you cannot
+use the C<add> or <addString> methods, or open another member with
+C<openMember>.
+
+You can then open another member with C<openMember> or use the C<add> or
+<addString> methods.
+
+Returns a filehandle on success or C<undef> on failure.
 
 =item $z->close() 
 
-Returns 1 if the zip archive was closed successfully, or 0. Check the $SimpleZipError for a message.
+Returns 1 if the zip archive was closed successfully, or 0. Check the
+$SimpleZipError for a message.
 
 =back 
 
 =head1 Options
 
-The majority of options are valid in both the constructor and in the methods that 
-accept options. Any exceptions are noted in the text below.
+The majority of options are valid in both the constructor and in the
+methods that accept options. Any exceptions are noted in the text below.
 
-Options specified in the constructor will be used as the defaults for all subsequent method call.
+Options specified in the constructor will be used as the defaults for all
+subsequent method calls.
 
-For example, in the constructor below, the C<Method> is set to C<ZIP_CM_STORE>. 
+For example, in the constructor below, the C<Method> is set to
+C<ZIP_CM_STORE>. 
 
     use Archive::Zip::SimpleZip qw($SimpleZipError) ;
 
@@ -502,19 +650,23 @@ For example, in the constructor below, the C<Method> is set to C<ZIP_CM_STORE>.
                              Method => ZIP_CM_STORE 
         or die "Cannot create zip file: $SimpleZipError\n" ;
 
-    $z->add("file1", Method => ZIP_CM_DEFLATE);
-    $z->add("file2");
+    $z->add("file1");
+    $z->add("file2", Method => ZIP_CM_DEFLATE);
+    $z->add("file3");
 
     $z->close();
    
 
-The first call to C<add> overrides the new default to use C<ZIP_CM_DEFLATE>, while the second
-uses the value set in the constructor, C<ZIP_CM_STORE>. 
+The first call to C<add> doesn't specify the C<Method> option, so it uses
+the value from the constructor (C<ZIP_CM_STORE>). The second call overrides
+the default set in the constructor to use C<ZIP_CM_DEFLATE>. The third will
+revert to using the default, C<ZIP_CM_STORE>. 
     
 
 =head2 File Naming Options
 
-These options control how the names of the files are store in the zip archive.
+The options listed below control how the names of the files are store in
+the zip archive.
 
 =over 5
 
@@ -522,11 +674,10 @@ These options control how the names of the files are store in the zip archive.
 
 Stores the contents of C<$string> in the zip filename header field. 
 
-When used with the C<add> method, this option will override any filename that
-was passed as a parameter.
+When used with the C<add> method, this option will override any filename
+that was passed as a parameter.
 
-
-The Name option is mandatory for the C<addString> method.
+The C<Name> option is mandatory for the C<addString> method.
 
 This option is not valid in the constructor.
 
@@ -562,43 +713,35 @@ filename is available C<$_> will contain an empty string.
 The value of C<$_> when the sub returns will be  stored in the filename
 header field.
 
-Note that if C<CanonicalName> is enabled, a
-normalized filename will be passed to the sub.
+Note that if C<CanonicalName> is enabled, a normalized filename will be
+passed to the sub.
 
 If you use C<FilterName> to modify the filename, it is your responsibility
 to keep the filename in Unix format.
 
-The example below shows how to FilterName can be use to remove the 
-path from the filename.
-
-    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
-
-    my $z = new Archive::Zip::SimpleZip "my.zip"
-        or die "$SimpleZipError\n" ;
-
-    for ( </some/path/*.c> )
-    {
-        $z->add($_, FilterName => sub { s[^.*/][] }  ) 
-            or die "Cannot add '$_' to zip file: $SimpleZipError\n" ;
-    }
-
-    $z->close();
-
+See L</Rename whilst adding> for an example of how the C<FilterName> option
+can be used.
 
 =back
 
-The filename entry stored in a Zip archive is constructed as follows.
+Taking all the options described above, filename entry stored in a Zip
+archive is constructed as follows.
 
-The initial source for the filename entry that gets stored in the zip archive is the filename parameter supplied
-to the C<add> method, or the value supplied with the Name option to the C<addSTring> method. 
+The initial source for the filename entry that gets stored in the zip
+archive is the filename parameter supplied to the C<add> method, or the
+value supplied with the C<Name> option to the C<addString> and
+C<openMember> methods. 
 
-Next, for the C<add> option, if the C<Name> option is supplied that will overide the filename parameter.
+Next, for the C<add> option, if the C<Name> option is supplied that will
+overide the filename parameter.
 
-If the C<CanonicalName> option is enabled, and it is by default, the filename gets normalized into Unix format. 
-If the filename was absolute, it will be changed into a relative filename.
+If the C<CanonicalName> option is enabled, and it is by default, the
+filename gets normalized into Unix format.  If the filename was absolute,
+it will be changed into a relative filename.
 
-Finally, is the C<FilterName> option is enabled, the filename will get passed to the sub supplied via the C<$_> variable. 
-The value of C<$_> on exit from the sub will get stored in the zip archive.
+Finally, is the C<FilterName> option is enabled, the filename will get
+passed to the sub supplied via the C<$_> variable.  The value of C<$_> on
+exit from the sub will get stored in the zip archive.
 
 Here are some examples
 
@@ -619,7 +762,7 @@ Here are some examples
     # store "file3.txt" in the zip archive
     $z->add("/my/file3.txt", FilterName => sub { s#.*/## } ) ;
         
-    # store "" in the zip archive
+    # no Name option, so store "" in the zip archive
     $z->addString("payload data") ;
             
     # store "xyz" in the zip archive
@@ -633,48 +776,48 @@ Here are some examples
 
 =head2 Overall Zip Archive Structure
 
-
-
 =over 5
 
 =item C<< Minimal => 1|0 >>
 
-If specified, this option will disable the creation of all extra fields
-in the zip local and central headers (with the exception of those needed for Zip64). 
+If specified, this option will disable the automatic creation of all
+I<extra> fields in the zip local and central headers (with the exception of
+those needed for Zip64). 
+
 In particular the following fields will not be created
 
-   Extended Time
-   TODO
+    "UT" Extended Timestamp
+    "ux" ExtraExtra Type 3 (if running Unix)
+
 
 This option is useful in a number of scenarios. 
 
-Firstly, the option may be needed if you require the zip files created 
-by C<Archive::Zip::SimpleZip> to be
-read using a legacy version of unzip or by an application that only supports
-a sub-set of the zip features.
+Firstly, it may be needed if you require the zip files created by
+C<Archive::Zip::SimpleZip> to be read using a legacy version of unzip or by
+an application that only supports a sub-set of the zip features.
 
-The other main use-case when C<Minimal> is handy is when the data that C<Minimal> suppresses
-is not needed, and so just adds unnecessary bloat to the zip file. 
-The extra fields that C<Archive::Zip::SimpleZip> adds by default all
-store additional information that assume the entry in the zip file corresponds to a file
-that will be stored in a filesystem.
-This information is very handy when archiving files from a filesystem - it means the unzipped files
-will more closely match their originals.
-If the zip file isn't going to be unzipped to a filesystem you can save a few bytes by enabling <Minimal>.
+The other main use-case when C<Minimal> is handy is when the data that
+C<Minimal> suppresses is not needed, and so just adds unnecessary bloat to
+the zip file.  The extra fields that C<Archive::Zip::SimpleZip> adds by
+default all store information that assume the entry in the zip file
+corresponds to a file that will be stored in a filesystem.  This
+information is very useful when archiving files from a filesystem - it
+means the unzipped files will more closely match their originals.  If the
+zip file isn't going to be unzipped to a filesystem you can save a few
+bytes by enabling <Minimal>.
 
 This parameter defaults to 0.
 
 =item C<< Stream => 0|1 >>
 
-This option controls whether the zip farchive is created in
-streaming mode.
+This option controls whether the zip archive is created in I<streaming
+mode>.
 
-Note that when outputting to a file or filehandle with streaming mode disabled (C<Stream>
-is 0), the output file/handle I<must> be seekable.
+Note that when outputting to a file or filehandle with streaming mode
+disabled (C<Stream> is 0), the output file/handle I<must> be seekable.
 
-When outputting to '-' (STDOUT) C<Stream> is automatically enabled.
-
-TODO - when to use & interoperability issues.
+When outputting to '-' (STDOUT) the C<Stream> option is automatically
+enabled.
 
 The default is 0.
 
@@ -690,12 +833,14 @@ ZIP64 is an extension to the Zip archive structure that allows
 
 =back
 
-The module will automatically enable ZIP64 mode as needed when creating zip archive.  
+The module will automatically enable ZIP64 mode as needed when creating zip
+archive.  
 
 You can force creation of a Zip64 zip archive by enabling this option.
 
 If you intend to manipulate the Zip64 zip archives created with this module
-using an external zip/unzip program/library, make sure that it supports Zip64.   
+using an external zip/unzip program/library, make sure that it supports
+Zip64.   
 
 The default is 0.
 
@@ -708,21 +853,39 @@ The default is 0.
 =item C<< Comment => $comment >>
 
 This option allows the creation of a comment that is associated with the
-entry added to the zip archive with the C<add> and C<addString> methods. 
+member added to the zip archive with the C<add> and C<addString> methods. 
 
 This option is not valid in the constructor.
 
 By default, no comment field is written to the zip archive.
 
+=item C<< Encode => "encoding" >>
+
+The C<Encode> option allows you to set the character encoding of the data
+before it is compressed and written to the zip file. The option is only
+valid with the C<add> or C<addString> methods. It will be ignored if you
+use it with the C<add> option.
+
+Under the hood this option relies on the C<Encode> module to carry do the
+hard work. In particular it uses the C<Encode::find_encoding> to check that
+the encoding you have request exists.
+   
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $z = new Archive::Zip::SimpleZip "my.zip"
+        or die "$SimpleZipError\n" ;
+       
+    $z->addString("payload data", Encode => "utf8") ;
+
 
 =item C<< Method => $method >>
 
 Controls which compression method is used. At present four compression
-methods are supported, namely Store (no compression at all), Deflate, 
+methods are supported, namely, Store (no compression at all), Deflate,
 Bzip2 and Lzma.
 
-The symbols, ZIP_CM_STORE, ZIP_CM_DEFLATE, ZIP_CM_BZIP2 and ZIP_CM_LZMA 
-are used to select the compression method.
+The symbols, ZIP_CM_STORE, ZIP_CM_DEFLATE, ZIP_CM_BZIP2 and ZIP_CM_LZMA are
+used to select the compression method.
 
 These constants are not imported by default by this module.
 
@@ -734,20 +897,22 @@ Note that to create Bzip2 content, the module C<IO::Compress::Bzip2> must
 be installed. A fatal error will be thrown if you attempt to create Bzip2
 content when C<IO::Compress::Bzip2> is not available.
 
-Note that to create Lzma content, the module C<IO::Compress::Lzma> must
-be installed. A fatal error will be thrown if you attempt to create Lzma
+Note that to create Lzma content, the module C<IO::Compress::Lzma> must be
+installed. A fatal error will be thrown if you attempt to create Lzma
 content when C<IO::Compress::Lzma> is not available.
 
-The default method is ZIP_CM_DEFLATE for files and ZIP_CM_STORE for directories and symbolic links.
+The default method is ZIP_CM_DEFLATE for files and ZIP_CM_STORE for
+directories and symbolic links.
 
 
 
 =item C<< StoreLink => 1|0  >>
 
-Controls what C<Archive::Zip::SimpleZip> does with a symbolic link.
+Controls what C<Archive::Zip::SimpleZip> does with a symbolic link
+(assuming your operating system supports .
 
-When true, it stores the link itself.
-When false, it stores the contents of the file the link refers to.
+When true, it stores the link itself.  When false, it stores the contents
+of the file the link refers to.
 
 If your platform does not support symbolic links this option is ignored.
 
@@ -758,15 +923,15 @@ Default is 0.
 =item C<< TextFlag => 0|1 >>
 
 This parameter controls the setting of a flag in the zip central header. It
-is used to signal that the data stored in the zip archive is probably
-text.
+is used to signal that the data stored in the zip archive is probably text.
 
 The default is 0. 
         
 
 =item C<< ZipComment => $comment >>
 
-This option allows the creation of a comment field for the entire zip archive.
+This option allows the creation of a comment field for the entire zip
+archive.
 
 This option is only valid in the constructor.
 
@@ -778,15 +943,15 @@ By default, no comment field is written to the zip archive.
 
 =head2 Deflate Compression Options
 
-These option are only valid if the C<Method> is ZIP_CM_DEFLATE. They are ignored
-otherwise.
+These option are only valid if the C<Method> is ZIP_CM_DEFLATE. They are
+ignored otherwise.
 
 =over 5
 
 =item C<< Level => value >> 
 
-Defines the compression level used by zlib. The value should either be
-a number between 0 and 9 (0 means no compression and 9 is maximum
+Defines the compression level used by zlib. The value should either be a
+number between 0 and 9 (0 means no compression and 9 is maximum
 compression), or one of the symbolic constants defined below.
 
    Z_NO_COMPRESSION
@@ -814,8 +979,8 @@ The default is Z_DEFAULT_STRATEGY.
 
 =head2 Bzip2 Compression Options
 
-These option are only valid if the C<Method> is ZIP_CM_BZIP2. They are ignored
-otherwise.
+These option are only valid if the C<Method> is ZIP_CM_BZIP2. They are
+ignored otherwise.
 
 =over 5
 
@@ -841,8 +1006,8 @@ The default is 0.
 
 =head2 Lzma Compression Options
 
-These option are only valid if the C<Method> is ZIP_CM_LZMA. They are ignored
-otherwise.
+These option are only valid if the C<Method> is ZIP_CM_LZMA. They are
+ignored otherwise.
 
 =over 5
 
@@ -877,7 +1042,7 @@ By default C<Archive::Zip::SimpleZip> will  do the following
 
 =item * Use Deflate Compression for all standard files. 
 
-=item * Create a non-streamed Zip archive
+=item * Create a non-streamed Zip archive.
 
 =item * Follow Symbolic Links
 
@@ -914,11 +1079,55 @@ Add all the "C" files in the current directory to the zip archive "my.zip".
     }
 
     $z->close();
+    
+=head2 Creating an in-memory Zip archive
 
+All you need to do if you want the zip archive to be created in-memory is
+to pass a string reference to the constructor.  The example below will
+store the zip archive in the variable C<$zipData>. 
 
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $zipData ;
+    my $z = new Archive::Zip::SimpleZip \$zipData
+        or die "$SimpleZipError\n" ;
+
+    $z->add("part1.txt");
+    $z->close(); 
+
+    
+Below is a slight refinement of the in-memory story. As well as writing the
+zip archive into memory, this example uses c<addString> to create the
+member "part2.txt" without having to read anything from the filesystem.
+
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $zipData ;
+    my $z = new Archive::Zip::SimpleZip \$zipData
+        or die "$SimpleZipError\n" ;
+
+    $z->addString("some text", Name => "part2.txt");
+    $z->close(); 
+
+       
 =head2 Rename whilst adding
 
-TODO
+The example below shows how the C<FilterName> option can be use to remove
+the path from the filename before it gets written to the zip archive,
+"my.zip".
+
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $z = new Archive::Zip::SimpleZip "my.zip"
+        or die "$SimpleZipError\n" ;
+
+    for ( </some/path/*.c> )
+    {
+        $z->add($_, FilterName => sub { s[^.*/][] }  ) 
+            or die "Cannot add '$_' to zip file: $SimpleZipError\n" ;
+    }
+
+    $z->close();
 
 =head2 Working with File::Find 
 
@@ -926,19 +1135,19 @@ TODO
 
 =head2 Writing a zip archive to a socket
 
-TODO
+TODO - use of the Stream option
 
 =head1 Importing 
 
-A number of symbolic constants are required by some methods in 
+A number of symbolic constants are required by some methods in
 C<Archive::Zip::SimpleZip>. None are imported by default.
 
 =over 5
 
 =item :all
 
-Imports C<zip>, C<$SimpleZipError> and all symbolic
-constants that can be used by C<IArchive::Zip::SimpleZip>. Same as doing this
+Imports C<zip>, C<$SimpleZipError> and all symbolic constants that can be
+used by C<IArchive::Zip::SimpleZip>. Same as doing this
 
     use Archive::Zip::SimpleZip qw(zip $SimpleZipError :constants) ;
 
@@ -961,7 +1170,8 @@ These symbolic constants are used by the C<flush> method.
 
 =item :level
 
-These symbolic constants are used by the C<Level> option in the constructor.
+These symbolic constants are used by the C<Level> option in the
+constructor.
 
     Z_NO_COMPRESSION
     Z_BEST_SPEED
@@ -970,7 +1180,8 @@ These symbolic constants are used by the C<Level> option in the constructor.
 
 =item :strategy
 
-These symbolic constants are used by the C<Strategy> option in the constructor.
+These symbolic constants are used by the C<Strategy> option in the
+constructor.
 
     Z_FILTERED
     Z_HUFFMAN_ONLY
@@ -989,6 +1200,54 @@ constructor.
 
 =back
 
+=head1 FAQ
+
+
+=head2 Can SimpleZip update an existing Zip file.
+
+No. You can only create a zip file from scratch.
+
+=head2 Can I write a Zip Archive to STDOUT?
+
+Yes. Writing zip files to filehandles that are not seekable (so that
+includes both STDOUT and sockets) is supported by this module. You just
+have to set the C<Stream> option when you call the constructor.
+
+    use Archive::Zip::SimpleZip qw($SimpleZipError) ;
+
+    my $zipData ;
+    my $z = new Archive::Zip::SimpleZip '-',
+                        Stream => 1
+        or die "$SimpleZipError\n" ;
+
+    $z->add("file1.txt");
+    $z->close(); 
+
+See L</What is a Streamed Zip file> for a discussion on the C<Stream>
+option.
+
+=head2 Can I write a Zip Archive directly to a socket?
+
+See previous question.
+
+=head2 What is a Streamed Zip file?
+
+Streaming mode allows you to write a zip file in situation where you cannot
+seek backwards/forwards. The classic examples are when you are working with
+sockets or need to write the zip file to STDOUT. 
+
+TODO - PUT EXAMPLE code that writes to a socket
+
+By default C<Archive::Zip::SimpleZip> does I<not> use streaming mode when
+writing to a zip file (you need to set the C<Stream> option to 1 to enable
+it). 
+
+If you plan to create a streamed Zip file be aware that it will be slightly
+larger than the non-streamed equivalent. If the files you archive are
+32-bit the overhead will be an extra bytes per file written to the zip
+archive. For 64-bit it is 24 bytes per file.
+
+
 =head1 SEE ALSO
 
 
@@ -1009,6 +1268,3 @@ Copyright (c) 2012 Paul Marquess. All rights reserved.
 
 This program is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
-
-    
-    
